@@ -40,6 +40,7 @@
 #include <sys/socket.h>
 #include <netdb.h>
 #include <sys/mman.h>
+#include <libgen.h>
 
 #include <http-parser/http_parser.h>
 
@@ -403,8 +404,8 @@ int stream_close(ngtcp2_conn *conn, uint32_t flags, int64_t stream_id,
 } // namespace
 
 namespace {
-int extend_max_streams_bidi(ngtcp2_conn *conn, uint64_t max_streams,
-                            void *user_data) {
+int extend_max_local_streams_bidi(ngtcp2_conn *conn, uint64_t max_streams,
+                                  void *user_data) {
   auto c = static_cast<Client *>(user_data);
 
   if (c->on_extend_max_streams() != 0) {
@@ -556,14 +557,7 @@ int recv_new_token(ngtcp2_conn *conn, const uint8_t *token, size_t tokenlen,
     return 0;
   }
 
-  auto f = BIO_new_file(config.token_file.data(), "w");
-  if (f == nullptr) {
-    std::cerr << "Could not write token in " << config.token_file << std::endl;
-    return 0;
-  }
-
-  PEM_write_bio(f, "QUIC TOKEN", "", token, tokenlen);
-  BIO_free(f);
+  util::write_token(config.token_file, token, tokenlen);
 
   return 0;
 }
@@ -616,8 +610,8 @@ int Client::init(int fd, const Address &local_addr, const Address &remote_addr,
       stream_close,
       nullptr, // recv_stateless_reset
       ngtcp2_crypto_recv_retry_cb,
-      extend_max_streams_bidi,
-      nullptr, // extend_max_streams_uni
+      extend_max_local_streams_bidi,
+      nullptr, // extend_max_local_streams_uni
       rand,
       get_new_connection_id,
       nullptr, // remove_connection_id
@@ -737,6 +731,7 @@ int Client::init(int fd, const Address &local_addr, const Address &remote_addr,
   params.initial_max_streams_uni = 0;
   params.max_idle_timeout = config.timeout;
   params.active_connection_id_limit = 7;
+  params.grease_quic_bit = 1;
 
   auto path = ngtcp2_path{
       {
@@ -841,7 +836,7 @@ int Client::on_read(const Endpoint &ep) {
   msg.msg_iov = &msg_iov;
   msg.msg_iovlen = 1;
 
-  uint8_t msg_ctrl[CMSG_SPACE(sizeof(uint8_t)) + CMSG_SPACE(sizeof(uint16_t))];
+  uint8_t msg_ctrl[CMSG_SPACE(sizeof(int)) + CMSG_SPACE(sizeof(uint16_t))];
   msg.msg_control = msg_ctrl;
 
   for (;;) {
@@ -874,8 +869,8 @@ int Client::on_read(const Endpoint &ep) {
         std::cerr << "Received packet: local="
                   << util::straddr(&ep.addr.su.sa, ep.addr.len)
                   << " remote=" << util::straddr(&su.sa, msg.msg_namelen)
-                  << " ecn=0x" << std::hex << pi.ecn << std::dec << " "
-                  << datalen << " bytes" << std::endl;
+                  << " ecn=0x" << std::hex << static_cast<uint32_t>(pi.ecn)
+                  << std::dec << " " << datalen << " bytes" << std::endl;
       }
 
       if (debug::packet_lost(config.rx_loss_prob)) {
@@ -1192,6 +1187,7 @@ int create_sock(Address &remote_addr, const char *addr, const char *port) {
 
   remote_addr.len = rp->ai_addrlen;
   memcpy(&remote_addr.su, rp->ai_addr, rp->ai_addrlen);
+  remote_addr.ifindex = 0;
 
   return fd;
 }
@@ -1404,7 +1400,31 @@ int Client::send_packet(const Endpoint &ep, const ngtcp2_addr &remote_addr,
   msg.msg_iov = &msg_iov;
   msg.msg_iovlen = 1;
 
-  fd_set_ecn(ep.fd, remote_addr.addr->sa_family, ecn);
+  uint8_t msg_ctrl[CMSG_SPACE(sizeof(int))];
+
+  memset(msg_ctrl, 0, sizeof(msg_ctrl));
+
+  msg.msg_control = msg_ctrl;
+  msg.msg_controllen = sizeof(msg_ctrl);
+
+  auto cm = CMSG_FIRSTHDR(&msg);
+  cm->cmsg_len = CMSG_LEN(sizeof(int));
+  memcpy(CMSG_DATA(cm), &ecn, sizeof(ecn));
+
+  switch (remote_addr.addr->sa_family) {
+  case AF_INET:
+    cm->cmsg_level = IPPROTO_IP;
+    cm->cmsg_type = IP_TOS;
+
+    break;
+  case AF_INET6:
+    cm->cmsg_level = IPPROTO_IPV6;
+    cm->cmsg_type = IPV6_TCLASS;
+
+    break;
+  default:
+    assert(0);
+  }
 
   ssize_t nwrite = 0;
 
@@ -1792,9 +1812,13 @@ int parse_requests(char **argv, size_t argvlen) {
 
 std::ofstream keylog_file;
 
+namespace {
+const char *prog = "h09client";
+} // namespace
+
 namespace {
 void print_usage() {
-  std::cerr << "Usage: h09client [OPTIONS] <HOST> <PORT> [<URI>...]"
+  std::cerr << "Usage: " << prog << " [OPTIONS] <HOST> <PORT> [<URI>...]"
             << std::endl;
 }
 } // namespace
@@ -1815,7 +1839,6 @@ void config_set_default(Config &config) {
   config.http_method = "GET"sv;
   config.max_data = 15_m;
   config.max_stream_data_bidi_local = 6_m;
-  config.max_stream_data_bidi_remote = 6_m;
   config.max_stream_data_uni = 6_m;
   config.max_window = 24_m;
   config.max_stream_window = 16_m;
@@ -1966,11 +1989,13 @@ Options:
               Default: )"
             << util::format_uint_iec(config.max_stream_data_uni) << R"(
   --max-streams-bidi=<N>
-              The number of the concurrent bidirectional streams.
+              The number of the  concurrent bidirectional streams that
+              the remote endpoint initiates.
               Default: )"
             << config.max_streams_bidi << R"(
   --max-streams-uni=<N>
-              The number of the concurrent unidirectional streams.
+              The number of the concurrent unidirectional streams that
+              the remote endpoint initiates.
               Default: )"
             << config.max_streams_uni << R"(
   --exit-on-first-stream-close
@@ -1985,7 +2010,7 @@ Options:
               specified.
   --disable-early-data
               Disable early data.
-  --cc=(cubic|reno|bbr|bbr2)
+  --cc=(cubic|reno|bbr)
               The name of congestion controller algorithm.
               Default: )"
             << util::strccalgo(config.cc_algo) << R"(
@@ -2052,6 +2077,10 @@ int main(int argc, char **argv) {
   const char *private_key_file = nullptr;
   const char *cert_file = nullptr;
 
+  if (argc) {
+    prog = basename(argv[0]);
+  }
+
   for (;;) {
     static int flag = 0;
     constexpr static option long_opts[] = {
@@ -2358,11 +2387,7 @@ int main(int argc, char **argv) {
           config.cc_algo = NGTCP2_CC_ALGO_BBR;
           break;
         }
-        if (strcmp("bbr2", optarg) == 0) {
-          config.cc_algo = NGTCP2_CC_ALGO_BBR2;
-          break;
-        }
-        std::cerr << "cc: specify cubic, reno, bbr, or bbr2" << std::endl;
+        std::cerr << "cc: specify cubic, reno, or bbr" << std::endl;
         exit(EXIT_FAILURE);
       case 28:
         // --exit-on-all-streams-close
@@ -2566,13 +2591,15 @@ int main(int argc, char **argv) {
     }
     config.fd = fd;
     config.datalen = st.st_size;
-    auto addr = mmap(nullptr, config.datalen, PROT_READ, MAP_SHARED, fd, 0);
-    if (addr == MAP_FAILED) {
-      std::cerr << "data: Could not mmap file " << data_path << ": "
-                << strerror(errno) << std::endl;
-      exit(EXIT_FAILURE);
+    if (config.datalen) {
+      auto addr = mmap(nullptr, config.datalen, PROT_READ, MAP_SHARED, fd, 0);
+      if (addr == MAP_FAILED) {
+        std::cerr << "data: Could not mmap file " << data_path << ": "
+                  << strerror(errno) << std::endl;
+        exit(EXIT_FAILURE);
+      }
+      config.data = static_cast<uint8_t *>(addr);
     }
-    config.data = static_cast<uint8_t *>(addr);
   }
 
   auto addr = argv[optind++];
