@@ -41,8 +41,6 @@
 #include <ngtcp2/ngtcp2.h>
 #include <nghttp3/nghttp3.h>
 
-#include <ev.h>
-
 namespace ngtcp2 {
 
 namespace util {
@@ -95,14 +93,16 @@ std::string format_durationf(uint64_t ns);
 
 std::mt19937 make_mt19937();
 
-ngtcp2_tstamp timestamp(struct ev_loop *loop);
+ngtcp2_tstamp timestamp();
 
 bool numeric_host(const char *hostname);
 
 bool numeric_host(const char *hostname, int family);
 
-// Dumps |src| of length |len| in the format similar to `hexdump -C`.
-void hexdump(FILE *out, const uint8_t *src, size_t len);
+// hexdump dumps |data| of length |datalen| in the format similar to
+// hexdump(1) with -C option.  This function returns 0 if it succeeds,
+// or -1.
+int hexdump(FILE *out, const void *data, size_t datalen);
 
 inline char lowcase(char c) {
   constexpr static unsigned char tbl[] = {
@@ -217,15 +217,13 @@ read_mime_types(const std::string_view &filename);
 
 // format_uint converts |n| into string.
 template <typename T> std::string format_uint(T n) {
-  std::string res;
   if (n == 0) {
-    res = "0";
-    return res;
+    return "0";
   }
   size_t nlen = 0;
   for (auto t = n; t; t /= 10, ++nlen)
     ;
-  res.resize(nlen);
+  std::string res(nlen, '\0');
   for (; n; n /= 10) {
     res[--nlen] = (n % 10) + '0';
   }
@@ -338,6 +336,11 @@ std::optional<std::string> read_token(const std::string_view &filename);
 int write_token(const std::string_view &filename, const uint8_t *token,
                 size_t tokenlen);
 
+std::optional<std::string>
+read_transport_params(const std::string_view &filename);
+int write_transport_params(const std::string_view &filename,
+                           const uint8_t *data, size_t datalen);
+
 const char *crypto_default_ciphers();
 
 const char *crypto_default_groups();
