@@ -153,10 +153,10 @@ size_t write_pkt_flags(uint8_t *out, size_t outlen, uint8_t flags,
 }
 
 static size_t write_long_header_pkt_generic(
-    uint8_t *out, size_t outlen, uint8_t pkt_type, const ngtcp2_cid *dcid,
-    const ngtcp2_cid *scid, int64_t pkt_num, uint32_t version,
-    const uint8_t *token, size_t tokenlen, ngtcp2_frame *fr, size_t frlen,
-    ngtcp2_crypto_km *ckm) {
+    uint8_t *out, size_t outlen, uint8_t flags, uint8_t pkt_type,
+    const ngtcp2_cid *dcid, const ngtcp2_cid *scid, int64_t pkt_num,
+    uint32_t version, const uint8_t *token, size_t tokenlen, ngtcp2_frame *fr,
+    size_t frlen, ngtcp2_crypto_km *ckm) {
   ngtcp2_crypto_cc cc;
   ngtcp2_ppe ppe;
   ngtcp2_pkt_hd hd;
@@ -185,14 +185,14 @@ static size_t write_long_header_pkt_generic(
      pretend that it is QUIC v1 here and rewrite the version field
      later. */
   ngtcp2_pkt_hd_init(
-      &hd, NGTCP2_PKT_FLAG_LONG_FORM, pkt_type, dcid, scid, pkt_num, 4,
-      version != NGTCP2_PROTO_VER_V1 && version != NGTCP2_PROTO_VER_V2_DRAFT
+      &hd, NGTCP2_PKT_FLAG_LONG_FORM | flags, pkt_type, dcid, scid, pkt_num, 4,
+      version != NGTCP2_PROTO_VER_V1 && version != NGTCP2_PROTO_VER_V2
           ? NGTCP2_PROTO_VER_V1
           : version,
       0);
 
-  hd.token.base = (uint8_t *)token;
-  hd.token.len = tokenlen;
+  hd.token = token;
+  hd.tokenlen = tokenlen;
 
   ngtcp2_ppe_init(&ppe, out, outlen, &cc);
   rv = ngtcp2_ppe_encode_hd(&ppe, &hd);
@@ -209,31 +209,42 @@ static size_t write_long_header_pkt_generic(
   return (size_t)n;
 }
 
+size_t write_initial_pkt_flags(uint8_t *out, size_t outlen, uint8_t flags,
+                               const ngtcp2_cid *dcid, const ngtcp2_cid *scid,
+                               int64_t pkt_num, uint32_t version,
+                               const uint8_t *token, size_t tokenlen,
+                               ngtcp2_frame *fr, size_t frlen,
+                               ngtcp2_crypto_km *ckm) {
+  return write_long_header_pkt_generic(out, outlen, flags, NGTCP2_PKT_INITIAL,
+                                       dcid, scid, pkt_num, version, token,
+                                       tokenlen, fr, frlen, ckm);
+}
+
 size_t write_initial_pkt(uint8_t *out, size_t outlen, const ngtcp2_cid *dcid,
                          const ngtcp2_cid *scid, int64_t pkt_num,
                          uint32_t version, const uint8_t *token,
                          size_t tokenlen, ngtcp2_frame *fr, size_t frlen,
                          ngtcp2_crypto_km *ckm) {
-  return write_long_header_pkt_generic(out, outlen, NGTCP2_PKT_INITIAL, dcid,
-                                       scid, pkt_num, version, token, tokenlen,
-                                       fr, frlen, ckm);
+  return write_initial_pkt_flags(out, outlen, NGTCP2_PKT_FLAG_NONE, dcid, scid,
+                                 pkt_num, version, token, tokenlen, fr, frlen,
+                                 ckm);
 }
 
 size_t write_handshake_pkt(uint8_t *out, size_t outlen, const ngtcp2_cid *dcid,
                            const ngtcp2_cid *scid, int64_t pkt_num,
                            uint32_t version, ngtcp2_frame *fr, size_t frlen,
                            ngtcp2_crypto_km *ckm) {
-  return write_long_header_pkt_generic(out, outlen, NGTCP2_PKT_HANDSHAKE, dcid,
-                                       scid, pkt_num, version, NULL, 0, fr,
-                                       frlen, ckm);
+  return write_long_header_pkt_generic(
+      out, outlen, NGTCP2_PKT_FLAG_NONE, NGTCP2_PKT_HANDSHAKE, dcid, scid,
+      pkt_num, version, NULL, 0, fr, frlen, ckm);
 }
 
 size_t write_0rtt_pkt(uint8_t *out, size_t outlen, const ngtcp2_cid *dcid,
                       const ngtcp2_cid *scid, int64_t pkt_num, uint32_t version,
                       ngtcp2_frame *fr, size_t frlen, ngtcp2_crypto_km *ckm) {
-  return write_long_header_pkt_generic(out, outlen, NGTCP2_PKT_0RTT, dcid, scid,
-                                       pkt_num, version, NULL, 0, fr, frlen,
-                                       ckm);
+  return write_long_header_pkt_generic(out, outlen, NGTCP2_PKT_FLAG_NONE,
+                                       NGTCP2_PKT_0RTT, dcid, scid, pkt_num,
+                                       version, NULL, 0, fr, frlen, ckm);
 }
 
 ngtcp2_strm *open_stream(ngtcp2_conn *conn, int64_t stream_id) {
@@ -386,7 +397,7 @@ ngtcp2_ssize pkt_decode_hd_short_mask(ngtcp2_pkt_hd *dest, const uint8_t *pkt,
 static void addr_init(ngtcp2_sockaddr_in *dest, uint32_t addr, uint16_t port) {
   memset(dest, 0, sizeof(*dest));
 
-  dest->sin_family = AF_INET;
+  dest->sin_family = NGTCP2_AF_INET;
   dest->sin_port = ngtcp2_htons(port);
   dest->sin_addr.s_addr = ngtcp2_htonl(addr);
 }
